#include "StdAfx.h"

#include "ScriptExecutor.h"

#include "Ident.h"
#include "AbstractParseTree.h"
#include "TextFileBuffer.h"
#include "Scanner.h"
#include "AbstractParser.h"
#include "IParser.h"
#include <math.h>
#include "Sample.h"
#include "SampleMultiDocTemplate.h"
#include "SampleDoc.h"
#include "SampleImageDoc.h"
#include "gfl.h"


static int roundToInt(double v) { return (int)floor(v + 0.5); }


class DoubleColorValue
{
public:
	DoubleColorValue() : r(0.0), g(0.0), b(0.0) {}
	DoubleColorValue(double r_val, double g_val, double b_val) : r(r_val), g(g_val), b(b_val) {}
	double r;
	double g;
	double b;
};

class TextAccessor
{
public:
	TextAccessor() : _document(0), _textBuffer(0) {}
	void open(const char* name)
	{
		_document = 0;
		_textBuffer = 0;

		CSampleApp* app = (CSampleApp*)AfxGetApp();
		CSampleMultiDocTemplate* docTemplate = app->GetDocTemplate();

		// See if there is already a document with given name as file name or title
		{
			CString sName(name);

			POSITION pos = docTemplate->GetFirstDocPosition();
			while (pos != NULL)
			{
				CDocument* pDoc = docTemplate->GetNextDoc(pos);
				if (lstrcmpi(pDoc->GetPathName(), name) == 0 || pDoc->GetTitle() == sName)
				{
					CAbstractDocument* document = (CAbstractDocument*)pDoc;
					if (document->documentType() == 1)
					{
						_document = (CSampleDoc*)document;
						_textBuffer = &_document->m_xTextBuffer;
						_textView = _textBuffer->GetFirstView();
					}
					return;
				}
			}
		}

		CSampleDoc* pDocument = (CSampleDoc*)docTemplate->CreateNewDocument();

		BOOL bAutoDelete = pDocument->m_bAutoDelete;
		pDocument->m_bAutoDelete = FALSE;   // don't destroy if something goes wrong
		CFrameWnd* pFrame = docTemplate->CreateNewFrame(pDocument, NULL);
		pDocument->m_bAutoDelete = bAutoDelete;
		if (pFrame == NULL)
		{
			AfxMessageBox(AFX_IDP_FAILED_TO_CREATE_DOC);
			delete pDocument;       // explicit delete on error
			return;
		}
		ASSERT_VALID(pFrame);

		// open an existing document
		CWaitCursor wait;
		if (!GFLBitmap::isImageFormat(name))
		{
			pDocument->setSilent();
			if (!pDocument->OnOpenDocument(name))
			{
				pDocument->OnNewDocument();
			}
		}
		pDocument->SetPathName(name);

		docTemplate->InitialUpdateFrame(pFrame, pDocument, true);
		_document = pDocument;
		_textBuffer = &_document->m_xTextBuffer;
		_textView = _textBuffer->GetFirstView();
	}

	void clear()
	{
		if (_textBuffer == 0)
			return;
	}

	void appendText(const char* s)
	{
		if (_textBuffer == 0)
			return;

		int lastLine = _textBuffer->GetLineCount()-1;
		int lastPos = _textBuffer->GetLineLength(lastLine);
		int endLine;
		int endChar;
		_textBuffer->InsertText(_textView, lastLine, lastPos, s, endLine, endChar);
	}

private:
	CSampleDoc* _document;
	CCrystalTextBuffer* _textBuffer;
	CCrystalTextView* _textView;
};

class ImageAccessor
{
public:
	ImageAccessor() : _imageDocument(0), _gflBitmap(0) {}
	void open(const char* name)
	{
		if (!GFLBitmap::initialized())
		{
			AfxMessageBox(_T("libgfl311.dll not loaded."));
			return;
		}

		_imageDocument = 0;

		CSampleApp* app = (CSampleApp*)AfxGetApp();
		CSampleMultiDocTemplate* docTemplate = app->GetDocTemplate();

		// See if there is already a document with given name as file name or title
		{
			CString sName(name);

			POSITION pos = docTemplate->GetFirstDocPosition();
			while (pos != NULL)
			{
				CDocument* pDoc = docTemplate->GetNextDoc(pos);
				if (lstrcmpi(pDoc->GetPathName(), name) == 0 || pDoc->GetTitle() == sName)
				{
					CAbstractDocument* document = (CAbstractDocument*)pDoc;
					if (document->documentType() == 2)
					{
						_imageDocument = (CSampleImageDoc*)document;
						_gflBitmap = &_imageDocument->m_gflBitmap;
					}
					return;
				}
			}
		}

		CSampleImageDoc* pDocument = docTemplate->createDocImageObject();
		ASSERT_KINDOF(CDocument, pDocument);
		docTemplate->AddDocument(pDocument);

		BOOL bAutoDelete = pDocument->m_bAutoDelete;
		pDocument->m_bAutoDelete = FALSE;   // don't destroy if something goes wrong
		CFrameWnd* pFrame = docTemplate->CreateNewImageFrame(pDocument, NULL);
		pDocument->m_bAutoDelete = bAutoDelete;
		if (pFrame == NULL)
		{
			AfxMessageBox(AFX_IDP_FAILED_TO_CREATE_DOC);
			delete pDocument;       // explicit delete on error
			return;
		}
		ASSERT_VALID(pFrame);

		// open an existing document
		CWaitCursor wait;
		if (GFLBitmap::isImageFormat(name))
		{
			pDocument->setSilent();
			if (!pDocument->OnOpenDocument(name))
			{
				pDocument->OnNewDocument();
			}
		}
		pDocument->SetPathName(name);

		docTemplate->InitialUpdateFrame(pFrame, pDocument, true);
		_imageDocument = pDocument;
		_gflBitmap = &_imageDocument->m_gflBitmap;
	}

	int width()
	{
		return _gflBitmap != 0 ? _gflBitmap->width() : 0;
	}

	int height()
	{
		return _gflBitmap != 0 ? _gflBitmap->height() : 0;
	}

	void newImage(int width, int height)
	{
		if (_gflBitmap == 0)
			return;

		_gflBitmap->newImage(width, height);
	}

	DoubleColorValue getInterpolatedColor(double x, double y)
	{
		if (_gflBitmap == 0)
			return DoubleColorValue();

		double r, g, b;
		_gflBitmap->getInterpolatedColor(x, y, r, g, b);
		return DoubleColorValue(r, g, b);
	}

	void setColor(int x, int y, const DoubleColorValue& color)
	{
		if (_gflBitmap == 0)
			return;
	
		int r = roundToInt(color.r);
		int g = roundToInt(color.g);
		int b = roundToInt(color.b);
		_gflBitmap->setColor(x, y, r, g, b);
	}

private:
	CSampleImageDoc* _imageDocument;
	GFLBitmap* _gflBitmap;
};



class Expr
{
public:
	enum TypeEnum { typeNone, typeVoid,
					typeBool, typeBoolVar, typeBoolArray,
					typeInt, typeIntVar, typeIntArray,
					typeDouble, typeDoubleVar, typeDoubleArray,
					typeString,
					typeImage, typeImageVar,
					typeColor, typeColorVar, typeColorArray,
					typeDoubleColor, typeDoubleColorVar, typeDoubleColorArray };
	virtual ~Expr() {}
	virtual TypeEnum type() { return typeNone; }
	virtual bool isVar() { return false; }
};

class VoidExpr : public Expr
{
public:
	TypeEnum type() { return typeVoid; }
	virtual void eval() {}
};

class BoolExpr : public Expr
{
public:
	TypeEnum type() { return typeBool; }
	virtual bool eval() { return 0; }
};

class IntExpr : public Expr
{
public:
	TypeEnum type() { return typeInt; }
	virtual int eval() { return 0; }
};

class DoubleExpr : public Expr
{
public:
	TypeEnum type() { return typeDouble; }
	virtual double eval() { return 0; }
};

class StringExpr : public Expr
{
public:
	TypeEnum type() { return typeString; }
	virtual const char* eval() { return ""; }
};

class DoubleColorExpr : public Expr
{
public:
	TypeEnum type() { return typeDoubleColor; }
	virtual const DoubleColorValue& eval() { return _value; }
protected:
	DoubleColorValue _value;
};

//////////////// BoolExpr ////////////////

class ConstBoolExpr : public BoolExpr
{
public:
	ConstBoolExpr(bool value) : _value(value) {}
	bool eval() { return _value; }
private:
	bool _value;
};

class NegateBoolExpr : BoolExpr
{
public: 
	NegateBoolExpr(BoolExpr* rhs) : _rhs(rhs) {}
	virtual ~NegateBoolExpr() { delete _rhs; }
	bool eval() { return !_rhs->eval(); }
private:
	BoolExpr* _rhs;
};

class DiadicBoolExpr : public BoolExpr
{
public:
	DiadicBoolExpr(BoolExpr* lhs, BoolExpr* rhs) : _lhs(lhs), _rhs(rhs) {}
	virtual ~DiadicBoolExpr() { delete _lhs; delete _rhs; }
protected:
	BoolExpr* _lhs;
	BoolExpr* _rhs;
};

#define CLASSDIADIC(C,T,RT,O) \
class C##T : public Diadic##T \
{\
public:\
	C##T(T* lhs, T* rhs) : Diadic##T(lhs, rhs) {}\
	RT eval() { return _lhs->eval() O _rhs->eval();}\
};

CLASSDIADIC(And, BoolExpr, bool, &&)
CLASSDIADIC(Or, BoolExpr, bool, ||)


class DiadicCompareIntExpr : public BoolExpr
{
public:
	DiadicCompareIntExpr(IntExpr* lhs, IntExpr* rhs) : _lhs(lhs), _rhs(rhs) {}
	virtual ~DiadicCompareIntExpr() { delete _lhs; delete _rhs; }
protected:
	IntExpr* _lhs;
	IntExpr* _rhs;
};

#define CLASSCOMPARE(C,T,O) \
class C##T##BoolExpr : public DiadicCompare##T##Expr \
{\
public:\
	C##T##BoolExpr(T##Expr* lhs, T##Expr* rhs) : DiadicCompare##T##Expr(lhs, rhs) {}\
	bool eval() { return _lhs->eval() O _rhs->eval();}\
};

CLASSCOMPARE(Equal, Int, ==)
CLASSCOMPARE(Unequal, Int, !=)
CLASSCOMPARE(Smaller, Int, <)
CLASSCOMPARE(SmallerOrEqual, Int, <=)
CLASSCOMPARE(Larger, Int, >)
CLASSCOMPARE(LargerOrEqual, Int, >=)

class DiadicCompareDoubleExpr : public BoolExpr
{
public:
	DiadicCompareDoubleExpr(DoubleExpr* lhs, DoubleExpr* rhs) : _lhs(lhs), _rhs(rhs) {}
	virtual ~DiadicCompareDoubleExpr() { delete _lhs; delete _rhs; }
protected:
	DoubleExpr* _lhs;
	DoubleExpr* _rhs;
};

CLASSCOMPARE(Equal, Double, ==)
CLASSCOMPARE(Unequal, Double, !=)
CLASSCOMPARE(Smaller, Double, <)
CLASSCOMPARE(SmallerOrEqual, Double, <=)
CLASSCOMPARE(Larger, Double, >)
CLASSCOMPARE(LargerOrEqual, Double, >=)

class VarBoolDecl : public VoidExpr
{
public:
	VarBoolDecl(BoolExpr* lhs) : _lhs(lhs) {}
	virtual ~VarBoolDecl() { delete _lhs; }
	TypeEnum type() { return typeBoolVar; }
	void eval()
	{
		_var = _lhs != 0 ? _lhs->eval() : false;
	}
	bool& ref() { return _var; }
private:
	bool _var;
	BoolExpr* _lhs;
};

class VarBoolExpr : public BoolExpr
{
public:
	VarBoolExpr(bool& var) : _var(var) {}
	bool isVar() { return true; }
	bool eval() { return _var; }
	void assign(bool value) { _var = value; }
private:
	bool& _var;
};

//////////////// IntExpr ////////////////

class ConstIntExpr : public IntExpr
{
public:
	ConstIntExpr(int value) : _value(value) {}
	int eval() { return _value; }
private:
	int _value;
};

class NegateIntExpr : IntExpr
{
public: 
	NegateIntExpr(IntExpr* rhs) : _rhs(rhs) {}
	virtual ~NegateIntExpr() { delete _rhs; }
	int eval() { return -_rhs->eval(); }
private:
	IntExpr* _rhs;
};

class DiadicIntExpr : public IntExpr
{
public:
	DiadicIntExpr(IntExpr* lhs, IntExpr* rhs) : _lhs(lhs), _rhs(rhs) {}
	virtual ~DiadicIntExpr() { delete _lhs; delete _rhs; }
protected:
	IntExpr* _lhs;
	IntExpr* _rhs;
};

class FloorFunctionIntExpr : public IntExpr
{
public:
	FloorFunctionIntExpr(DoubleExpr* arg) : _arg(arg) {}
	virtual ~FloorFunctionIntExpr() { delete _arg; }
	int eval() { return (int)floor(_arg->eval()); }
private:
	DoubleExpr* _arg;
};

CLASSDIADIC(Add, IntExpr, int, +)
CLASSDIADIC(Sub, IntExpr, int, -)
CLASSDIADIC(Times, IntExpr, int, *)
CLASSDIADIC(Div, IntExpr, int, /)
CLASSDIADIC(Mod, IntExpr, int, %)

class VarIntDecl : public VoidExpr
{
public:
	VarIntDecl(IntExpr* lhs) : _lhs(lhs) {}
	virtual ~VarIntDecl() { delete _lhs; }
	TypeEnum type() { return typeIntVar; }
	void eval()
	{
		_var = _lhs != 0 ? _lhs->eval() : false;
	}
	int& ref() { return _var; }
protected:
	int _var;
	IntExpr* _lhs;
};

class VarIntExpr : public IntExpr
{
public:
	VarIntExpr(int& var) : _var(var) {}
	bool isVar() { return true; }
	int eval() { return _var; }
	void assign(int value) { _var = value; }
private:
	int& _var;
};

//////////////// DoubleExpr ////////////////

class ConstDoubleExpr : public DoubleExpr
{
public:
	ConstDoubleExpr(double value) : _value(value) {}
	double eval() { return _value; }
private:
	double _value;
};

class ToDoubleExpr : public DoubleExpr
{
public: 
	ToDoubleExpr(IntExpr* rhs) : _rhs(rhs) {}
	virtual ~ToDoubleExpr() { delete _rhs; }
	double eval() { return _rhs->eval(); }
private:
	IntExpr* _rhs;
};

class NegateDoubleExpr : public DoubleExpr
{
public: 
	NegateDoubleExpr(DoubleExpr* rhs) : _rhs(rhs) {}
	virtual ~NegateDoubleExpr() { delete _rhs; }
	double eval() { return -_rhs->eval(); }
private:
	DoubleExpr* _rhs;
};

class DiadicDoubleExpr : public DoubleExpr
{
public:
	DiadicDoubleExpr(DoubleExpr* lhs, DoubleExpr* rhs) : _lhs(lhs), _rhs(rhs) {}
	virtual ~DiadicDoubleExpr() { delete _lhs; delete _rhs; }
protected:
	DoubleExpr* _lhs;
	DoubleExpr* _rhs;
};

CLASSDIADIC(Add, DoubleExpr, double, +)
CLASSDIADIC(Sub, DoubleExpr, double, -)
CLASSDIADIC(Times, DoubleExpr, double, *)
CLASSDIADIC(Div, DoubleExpr, double, /)

class VarDoubleDecl : public VoidExpr
{
public:
	VarDoubleDecl(DoubleExpr* lhs) : _lhs(lhs) {}
	virtual ~VarDoubleDecl() { delete _lhs; }
	TypeEnum type() { return typeDoubleVar; }
	void eval()
	{
		_var = _lhs != 0 ? _lhs->eval() : false;
	}
	double& ref() { return _var; }
private:
	double _var;
	DoubleExpr* _lhs;
};

class VarDoubleExpr : public DoubleExpr
{
public:
	VarDoubleExpr(double& var) : _var(var) {}
	bool isVar() { return true; }
	double eval() { return _var; }
	void assign(bool value) { _var = value; }
private:
	double& _var;
};

//////////////// StringExpr ////////////////

class ConstStringExpr : public StringExpr
{
public:
	ConstStringExpr(const char* value)
	{
		_value = new char[strlen(value)+1];
		strcpy(_value, value);
	}
	virtual ~ConstStringExpr()
	{
		delete _value;
	}
	const char* eval() { return _value; }
private:
	char* _value;
};

//////////////// ImageExpr ////////////////

class VarImageDecl : public VoidExpr
{
public:
	TypeEnum type() { return typeImageVar; }
	ImageAccessor& ref() { return _var; }
private:
	ImageAccessor _var;
};

class VarImageExpr : public Expr
{
public:
	VarImageExpr(ImageAccessor& var) : _var(var) {}
	TypeEnum type() { return typeImage; }
	bool isVar() { return true; }
	ImageAccessor& ref() { return _var; }
private:
	ImageAccessor& _var;
};

class ImageOpenMethodExpr : public VoidExpr
{
public:
	ImageOpenMethodExpr(VarImageExpr* varImageExpr, StringExpr* nameExpr) : _varImageExpr(varImageExpr), _nameExpr(nameExpr) {}
	void eval()
	{
		_varImageExpr->ref().open(_nameExpr->eval());
	}
private:
	VarImageExpr* _varImageExpr;
	StringExpr* _nameExpr;
};

class ImageNewMethodExpr : public VoidExpr
{
public:
	ImageNewMethodExpr(VarImageExpr* varImageExpr, IntExpr* widthExpr, IntExpr* heightExpr) : _varImageExpr(varImageExpr), _widthExpr(widthExpr), _heightExpr(heightExpr) {}
	void eval()
	{
		_varImageExpr->ref().newImage(_widthExpr->eval(), _heightExpr->eval());
	}
private:
	VarImageExpr* _varImageExpr;
	IntExpr* _widthExpr;
	IntExpr* _heightExpr;
};

class ImageGetInterpolatedColourMethodExpr : public DoubleColorExpr
{
public:
	ImageGetInterpolatedColourMethodExpr(VarImageExpr* varImageExpr, IntExpr* xExpr, IntExpr* yExpr) : _varImageExpr(varImageExpr), _xExpr(xExpr), _yExpr(yExpr) {}
	const DoubleColorValue& eval()
	{
		_value = _varImageExpr->ref().getInterpolatedColor(_xExpr->eval(), _yExpr->eval());
		return _value;
	}
private:
	VarImageExpr* _varImageExpr;
	IntExpr* _xExpr;
	IntExpr* _yExpr;
};

class ImageSetColourMethodExpr : public VoidExpr
{
public:
	ImageSetColourMethodExpr(VarImageExpr* varImageExpr, IntExpr* xExpr, IntExpr* yExpr, DoubleColorExpr* colorExpr) : _varImageExpr(varImageExpr), _xExpr(xExpr), _yExpr(yExpr), _colorExpr(colorExpr) {}
	void eval()
	{
		int x = _xExpr->eval();
		int y = _yExpr->eval();
		const DoubleColorValue& color = _colorExpr->eval();
		_varImageExpr->ref().setColor(x, y, color);
	}
private:
	VarImageExpr* _varImageExpr;
	DoubleColorValue _value;
	IntExpr* _xExpr;
	IntExpr* _yExpr;
	DoubleColorExpr* _colorExpr;
};

class ImageWidthFieldExpr : public IntExpr
{
public:
	ImageWidthFieldExpr(VarImageExpr* varImageExpr) : _varImageExpr(varImageExpr) {}
	int eval()
	{
		return _varImageExpr->ref().width();
	}
private:
	VarImageExpr* _varImageExpr;
};

class ImageHeightFieldExpr : public IntExpr
{
public:
	ImageHeightFieldExpr(VarImageExpr* varImageExpr) : _varImageExpr(varImageExpr) {}
	int eval()
	{
		return _varImageExpr->ref().height();
	}
private:
	VarImageExpr* _varImageExpr;
};

//////////////// DoubleColorExpr ////////////////

class VarDoubleColorDecl : public VoidExpr
{
public:
	VarDoubleColorDecl(DoubleColorExpr* lhs) : _lhs(lhs) {}
	TypeEnum type() { return typeDoubleColorVar; }
	void eval()
	{
		_var = _lhs != 0 ? _lhs->eval() : DoubleColorValue();
	}
	DoubleColorValue& ref() { return _var; }
private:
	DoubleColorValue _var;
	DoubleColorExpr* _lhs;
};

class VarDoubleColorExpr : public DoubleColorExpr
{
public:
	VarDoubleColorExpr(DoubleColorValue& var) : _var(var) {}
	bool isVar() { return true; }
	const DoubleColorValue& eval() { return _var; }
	DoubleColorValue& ref() { return _var; }
private:
	DoubleColorValue& _var;
};

//////////////// VoidExpr ////////////////

class StatementsExpr : public VoidExpr
{
public:
	StatementsExpr(int n) : _n(n)
	{
		_statements = _n > 0 ? new VoidExpr*[_n] : 0;
		for (int i = 0; i < _n; i++)
			_statements[i] = 0;
	}
	virtual ~StatementsExpr()
	{
		for (int i = 0; i < _n; i++)
			delete _statements[i];
		delete _statements;
	}
	void set(int i, VoidExpr* expr)
	{
		if (0 <= i && i < _n)
			_statements[i] = expr;
	}
	void eval()
	{
		for (int i = 0; i < _n; i++)
			if (_statements[i] != 0)
				_statements[i]->eval();
	}
private:
	int _n;
	VoidExpr** _statements;
};

class IfExpr : public VoidExpr
{
public:
	IfExpr(BoolExpr* condition, VoidExpr* then_part, VoidExpr* else_part) : _condition(condition), _then_part(then_part), _else_part(else_part) {}
	virtual ~IfExpr() { delete _condition; delete _then_part; delete _else_part; }
	void eval()
	{
		if (_condition->eval())
			_then_part->eval();
		else if (_else_part != 0)
			_else_part->eval();
	}
private:
	BoolExpr* _condition;
	VoidExpr* _then_part;
	VoidExpr* _else_part;
};

class ForIntExpr : public VoidExpr
{
public:
	ForIntExpr(VarIntDecl* varExpr, IntExpr* fromExpr, IntExpr* toExpr, VoidExpr* statement) : _varExpr(varExpr), _fromExpr(fromExpr), _toExpr(toExpr), _statement(statement) {}
	virtual ~ForIntExpr() { delete _varExpr; delete _fromExpr; delete _toExpr; delete _statement; }
	void setStatement(VoidExpr* statement) { _statement = statement; }
	void eval()
	{
		int& var = _varExpr->ref();
		var = _fromExpr->eval();
		int to = _toExpr->eval();
		for (; var < to; var++)
		{
			_statement->eval();
		}
	}
private:
	VarIntDecl* _varExpr;
	IntExpr* _fromExpr;
	IntExpr* _toExpr;
	VoidExpr* _statement;
};


//////////////// Context ////////////////

class Context
{
public:
	Context(Ident ident, VoidExpr* decl_expr, Context* next) : _ident(ident), _decl_expr(decl_expr), _next(next) {}
	VoidExpr* find(Ident ident)
	{
		if (ident == _ident)
			return _decl_expr;
		if (_next != 0)
			return _next->find(ident);
		return 0;
	}
private:
	Ident _ident;
	VoidExpr* _decl_expr;
	Context* _next;
};



void error(AbstractParseTree tree, const char* msg)
{
	CString prompt;
	prompt.Format("%d,%d: %s", tree.line(), tree.column(), msg);
	AfxMessageBox(prompt);
}

bool assertBoolType(Expr* expr, AbstractParseTree tree)
{
	if (expr != 0 && expr->type() == Expr::typeBool)
		return true;
	error(tree, "Expect bool value");
	return false;
}

bool assertIntType(Expr* expr, AbstractParseTree tree)
{
	if (expr != 0 && expr->type() == Expr::typeInt)
		return true;
	error(tree, "Expect int value");
	return false;
}

bool assertDoubleIntType(Expr* expr, AbstractParseTree tree)
{
	if (expr != 0 && (expr->type() == Expr::typeInt || expr->type() == Expr::typeDouble))
		return true;
	error(tree, "Expect int or double value");
	return false;
}

bool assertDoubleType(Expr* expr, AbstractParseTree tree)
{
	if (expr != 0 && expr->type() == Expr::typeDouble)
		return true;
	error(tree, "Expect int or double value");
	return false;
}

bool assertStringType(Expr* expr, AbstractParseTree tree)
{
	if (expr != 0 && expr->type() == Expr::typeString)
		return true;
	error(tree, "Expect string value");
	return false;
}

bool assertColorType(Expr* expr, AbstractParseTree tree)
{
	if (expr != 0 && (expr->type() == Expr::typeColor || expr->type() == Expr::typeDoubleColor))
		return true;
	error(tree, "Expect (double)color value");
	return false;
}

bool assertDoubleColorType(Expr* expr, AbstractParseTree tree)
{
	if (expr != 0 && expr->type() == Expr::typeDoubleColor)
		return true;
	error(tree, "Expect doublecolor value");
	return false;
}

Expr* compile(AbstractParseTree tree, Context* &context)
{
	if (tree.isInt())
		return new ConstIntExpr(tree.intValue());
	if (tree.isDouble())
		return new ConstDoubleExpr(tree.doubleValue());
	if (tree.isString())
		return new ConstStringExpr(tree.stringValue());
	if (tree.isIdent())
	{
		Expr* expr = context->find(tree.identName());
		if (expr == 0) { error(tree, "Variable not declared"); return new Expr(); }
		if (expr->type() == Expr::typeBoolVar)
		{
			VarIntDecl* varIntDecl = (VarIntDecl*)expr;
			return new VarIntExpr(varIntDecl->ref());
		}
		if (expr->type() == Expr::typeIntVar)
		{
			VarIntDecl* varIntDecl = (VarIntDecl*)expr;
			return new VarIntExpr(varIntDecl->ref());
		}
		if (expr->type() == Expr::typeDoubleVar)
		{
			VarDoubleDecl* varDoubleDecl = (VarDoubleDecl*)expr;
			return new VarDoubleExpr(varDoubleDecl->ref());
		}
		if (expr->type() == Expr::typeImageVar)
		{
			VarImageDecl* varImageDecl = (VarImageDecl*)expr;
			return new VarImageExpr(varImageDecl->ref());
		}
		if (expr->type() == Expr::typeDoubleColorVar)
		{
			VarDoubleColorDecl* varDoubleColorDecl = (VarDoubleColorDecl*)expr;
			return new VarDoubleColorExpr(varDoubleColorDecl->ref());
		}
		error(tree, "Type of var not implemented yet");
	}
	static Ident ident_statements("statements");
	if (tree.isTree(ident_statements))
	{
		Context* statement_context = context;
		AbstractParseTree statements = tree.part(1);
		ASSERT_ELSE(statements.isList()) return new Expr();
		StatementsExpr* stats_expr = new StatementsExpr(statements.nrParts());
		int i = 0;
		for (AbstractParseTree::iterator it(statements); it.more(); it.next())
		{
			Expr* expr = compile(it, statement_context);
			if (expr->type() == Expr::typeNone) 
				return new Expr();
			stats_expr->set(i++, (VoidExpr*)expr);
		}
		return stats_expr;
	}

	static Ident ident_for("for");
	if (tree.isTree(ident_for))
	{
		ASSERT_ELSE(tree.nrParts() == 4 && tree.part(1).isIdent()) return new Expr();
		Ident var = tree.part(1).identName();
		VarIntDecl* varIntDecl = new VarIntDecl(0);
		Context* for_context = new Context(var, varIntDecl, context);
		Expr* from_expr = compile(tree.part(2), context);
		if (!assertIntType(from_expr, tree.part(2))) return new Expr();
		Expr* to_expr = compile(tree.part(3), context);
		if (!assertIntType(to_expr, tree.part(3))) return new Expr();
		Expr* body = compile(tree.part(4), for_context);
		if (body->type() != Expr::typeVoid) return new Expr();
		return new ForIntExpr(varIntDecl, (IntExpr*)from_expr, (IntExpr*)to_expr, (VoidExpr*)body);
	}

	static Ident ident_if("if");
	if (tree.isTree(ident_if))
	{
		ASSERT_ELSE(tree.nrParts() == 3) return new Expr();
		Expr* condition = compile(tree.part(1), context);
		if (!assertBoolType(condition, tree.part(1))) return new Expr();
		Expr* then_part = compile(tree.part(2), context);
		if (then_part->type() != Expr::typeVoid) return new Expr();
		Expr* else_part = tree.part(3).isEmpty() ? 0 : compile(tree.part(3), context);
		if (else_part != 0 && else_part->type() != Expr::typeVoid) return new Expr();
		return new IfExpr((BoolExpr*)condition, (VoidExpr*)then_part, (VoidExpr*)else_part);
	}

	static Ident ident_vardecl("vardecl");
	if (tree.isTree(ident_vardecl))
	{
		ASSERT_ELSE(tree.nrParts() == 3 && tree.part(2).isIdent()) return new Expr();
		AbstractParseTree type = tree.part(1);
		Ident ident = tree.part(2).identName();
		Expr* init_expr = 0;
		if (!tree.part(3).isEmpty())
			init_expr = compile(tree.part(3), context);
		VoidExpr* decl_expr = 0;
		static Ident ident_bool("bool");
		if (type.isTree(ident_bool))
		{
			if (!assertBoolType(init_expr, tree.part(3))) return new Expr();
			decl_expr = new VarBoolDecl((BoolExpr*)init_expr);
		}
		static Ident ident_int("int");
		if (type.isTree(ident_int))
		{
			if (!assertIntType(init_expr, tree.part(3))) return new Expr();
			decl_expr = new VarIntDecl((IntExpr*)init_expr);
		}
		static Ident ident_double("double");
		if (type.isTree(ident_double))
		{
			if (!assertDoubleIntType(init_expr, tree.part(3))) return new Expr();
			decl_expr = new VarDoubleDecl(init_expr->type() == Expr::typeInt ? new ToDoubleExpr((IntExpr*)init_expr): (DoubleExpr*)init_expr);
		}
		static Ident ident_image("image");
		if (type.isTree(ident_image))
		{
			if (init_expr != 0) { error(tree.part(3), "Initialization not allowed"); return new Expr(); }
			decl_expr = new VarImageDecl();
		}
		static Ident ident_color("color");
		static Ident ident_doublecolor("doublecolor");
		if (type.isTree(ident_doublecolor))
		{
			if (!assertDoubleColorType(init_expr, tree.part(3))) return new Expr();
			decl_expr = new VarDoubleColorDecl((DoubleColorExpr*)init_expr);
		}

		if (decl_expr == 0)
		{
			error(tree, "Not implemented type");
			return new Expr();
		}
		context = new Context(ident, decl_expr, context);

		return decl_expr;
	}


	static Ident ident_add("add");
	static Ident ident_sub("sub");
	static Ident ident_times("times");
	static Ident ident_div("div");
	static Ident ident_mod("mod");
	static Ident ident_eq("eq");
	static Ident ident_ne("ne");
	static Ident ident_lt("lt");
	static Ident ident_le("le");
	static Ident ident_gt("gt");
	static Ident ident_ge("ge");
	if (   tree.isTree(ident_add)
		|| tree.isTree(ident_sub)
		|| tree.isTree(ident_times)
		|| tree.isTree(ident_div)
		|| tree.isTree(ident_mod)
		|| tree.isTree(ident_eq)
		|| tree.isTree(ident_ne)
		|| tree.isTree(ident_lt)
		|| tree.isTree(ident_le)
		|| tree.isTree(ident_gt)
		|| tree.isTree(ident_ge))
	{
		ASSERT_ELSE(tree.nrParts() == 2) return new Expr();
		Expr* lhs = compile(tree.part(1), context);
		if (!assertDoubleIntType(lhs, tree.part(1))) return new Expr();
		Expr* rhs = compile(tree.part(2), context);
		if (!assertDoubleIntType(rhs, tree.part(2))) return new Expr();
		if (lhs->type() == Expr::typeDouble || rhs->type() == Expr::typeDouble)
		{
			if (lhs->type() == Expr::typeInt)
				lhs = new ToDoubleExpr((IntExpr*)lhs);
			if (rhs->type() == Expr::typeInt)
				rhs = new ToDoubleExpr((IntExpr*)rhs);
			if (tree.isTree(ident_add))
				return new AddDoubleExpr((DoubleExpr*)lhs, (DoubleExpr*)rhs);
			if (tree.isTree(ident_sub))
				return new SubDoubleExpr((DoubleExpr*)lhs, (DoubleExpr*)rhs);
			if (tree.isTree(ident_times))
				return new TimesDoubleExpr((DoubleExpr*)lhs, (DoubleExpr*)rhs);
			if (tree.isTree(ident_div))
				return new DivDoubleExpr((DoubleExpr*)lhs, (DoubleExpr*)rhs);
			if (tree.isTree(ident_mod)) { error(tree, "% not allowed for double values"); return new Expr(); }
			if (tree.isTree(ident_eq))
				return new EqualDoubleBoolExpr((DoubleExpr*)lhs, (DoubleExpr*)rhs);
			if (tree.isTree(ident_ne))
				return new UnequalDoubleBoolExpr((DoubleExpr*)lhs, (DoubleExpr*)rhs);
			if (tree.isTree(ident_lt))
				return new SmallerDoubleBoolExpr((DoubleExpr*)lhs, (DoubleExpr*)rhs);
			if (tree.isTree(ident_le))
				return new SmallerOrEqualDoubleBoolExpr((DoubleExpr*)lhs, (DoubleExpr*)rhs);
			if (tree.isTree(ident_gt))
				return new LargerDoubleBoolExpr((DoubleExpr*)lhs, (DoubleExpr*)rhs);
			if (tree.isTree(ident_ge))
				return new LargerOrEqualDoubleBoolExpr((DoubleExpr*)lhs, (DoubleExpr*)rhs);
		}
		else
		{
			if (tree.isTree(ident_add))
				return new AddIntExpr((IntExpr*)lhs, (IntExpr*)rhs);
			if (tree.isTree(ident_sub))
				return new SubIntExpr((IntExpr*)lhs, (IntExpr*)rhs);
			if (tree.isTree(ident_times))
				return new TimesIntExpr((IntExpr*)lhs, (IntExpr*)rhs);
			if (tree.isTree(ident_div))
				return new DivIntExpr((IntExpr*)lhs, (IntExpr*)rhs);
			if (tree.isTree(ident_mod))
				return new ModIntExpr((IntExpr*)lhs, (IntExpr*)rhs);
			if (tree.isTree(ident_eq))
				return new EqualIntBoolExpr((IntExpr*)lhs, (IntExpr*)rhs);
			if (tree.isTree(ident_ne))
				return new UnequalIntBoolExpr((IntExpr*)lhs, (IntExpr*)rhs);
			if (tree.isTree(ident_lt))
				return new SmallerIntBoolExpr((IntExpr*)lhs, (IntExpr*)rhs);
			if (tree.isTree(ident_le))
				return new SmallerOrEqualIntBoolExpr((IntExpr*)lhs, (IntExpr*)rhs);
			if (tree.isTree(ident_gt))
				return new LargerIntBoolExpr((IntExpr*)lhs, (IntExpr*)rhs);
			if (tree.isTree(ident_ge))
				return new LargerOrEqualIntBoolExpr((IntExpr*)lhs, (IntExpr*)rhs);
		}
	}

	static Ident ident_and("and");
	static Ident ident_or("or");
	if (   tree.isTree(ident_and)
		|| tree.isTree(ident_or))
	{
		ASSERT_ELSE(tree.nrParts() == 2) return new Expr();
		Expr* lhs = compile(tree.part(1), context);
		if (!assertBoolType(lhs, tree.part(1))) return new Expr();
		Expr* rhs = compile(tree.part(2), context);
		if (!assertBoolType(rhs, tree.part(2))) return new Expr();
		if (tree.isTree(ident_and))
			return new AndBoolExpr((BoolExpr*)lhs, (BoolExpr*)rhs);
		if (tree.isTree(ident_or))
			return new OrBoolExpr((BoolExpr*)lhs, (BoolExpr*)rhs);
	}

	static Ident ident_field("field");
	if (tree.isTree(ident_field))
	{
		ASSERT_ELSE(tree.nrParts() == 2 && tree.part(2).isIdent()) return new Expr();
		Ident fieldName = tree.part(2).identName();
		Expr* target = compile(tree.part(1), context);
		if (!target->isVar()) { error(tree.part(1), "Target needs to be variable"); return new Expr(); }
		Expr* result = 0;
		if (target->type() == Expr::typeImage)
		{
			static Ident ident_width("width");
			if (fieldName == ident_width)
			{
				result = new ImageWidthFieldExpr((VarImageExpr*)target);
			}
			static Ident ident_height("height");
			if (fieldName == ident_height)
			{
				result = new ImageHeightFieldExpr((VarImageExpr*)target);
			}
		}
		if (result == 0)
		{
			error(tree, "unknown field");
			return new Expr();
		}
		return result;
	}

	static Ident ident_function("function");
	if (tree.isTree(ident_function))
	{
		ASSERT_ELSE(tree.nrParts() == 2 && tree.part(1).isIdent()) return new Expr();
		Ident functionName = tree.part(1).identName();
		AbstractParseTree arg_list = tree.part(2);
		int nr_args = arg_list.nrParts();
		Expr** arg_exprs = 0;
		if (nr_args > 0)
		{
			arg_exprs = new Expr*[nr_args];
			for (int i = 0; i < nr_args; i++)
				arg_exprs[i] = compile(arg_list.part(i+1), context);
		}
		Expr* result = 0;
		static Ident ident_floor("floor");
		if (functionName == ident_floor)
		{
			if (nr_args != 1) { error(arg_list, "Should have one argument"); return new Expr(); }
			if (!assertDoubleType(arg_exprs[0], arg_list)) return new Expr();
			result = new FloorFunctionIntExpr((DoubleExpr*)arg_exprs[0]);
		}
		if (result == 0)
		{
			error(tree, "Unknown function");
			for (int i = 0; i < nr_args; i++)
				delete arg_exprs[i];
			delete arg_exprs;
			return new Expr();
		}
		delete arg_exprs;
		return result;
	}

	static Ident ident_method("method");
	if (tree.isTree(ident_method))
	{
		ASSERT_ELSE(tree.nrParts() == 3 && tree.part(2).isIdent()) return new Expr();
		Ident methodName = tree.part(2).identName();
		Expr* target = compile(tree.part(1), context);
		if (!target->isVar()) { error(tree.part(1), "Target needs to be variable"); return new Expr(); }
		AbstractParseTree arg_list = tree.part(3);
		int nr_args = arg_list.nrParts();
		Expr** arg_exprs = 0;
		if (nr_args > 0)
		{
			arg_exprs = new Expr*[nr_args];
			for (int i = 0; i < nr_args; i++)
				arg_exprs[i] = compile(arg_list.part(i+1), context);
		}
		Expr* result = 0;
		if (target->type() == Expr::typeImage)
		{
			static Ident ident_open("open");
			if (methodName == ident_open)
			{
				if (nr_args != 1) { error(arg_list, "Should have one argument"); return new Expr(); }
				if (!assertStringType(arg_exprs[0], arg_list)) return new Expr();
				result = new ImageOpenMethodExpr((VarImageExpr*)target, (StringExpr*)arg_exprs[0]);
			}
			static Ident ident_new("new");
			if (methodName == ident_new)
			{
				if (nr_args != 2) { error(arg_list, "Should have two argument"); return new Expr(); }
				if (!assertIntType(arg_exprs[0], arg_list.part(1))) return new Expr();
				if (!assertIntType(arg_exprs[1], arg_list.part(2))) return new Expr();
				result = new ImageNewMethodExpr((VarImageExpr*)target, (IntExpr*)arg_exprs[0], (IntExpr*)arg_exprs[1]);
			}
			static Ident ident_getInterpolatedColour("getInterpolatedColour");
			if (methodName == ident_getInterpolatedColour)
			{
				if (nr_args != 2) { error(arg_list, "Should have two argument"); return new Expr(); }
				if (!assertIntType(arg_exprs[0], arg_list.part(1))) return new Expr();
				if (!assertIntType(arg_exprs[1], arg_list.part(2))) return new Expr();
				result = new ImageGetInterpolatedColourMethodExpr((VarImageExpr*)target, (IntExpr*)arg_exprs[0], (IntExpr*)arg_exprs[1]);
			}
			static Ident ident_setColour("setColour");
			if (methodName == ident_setColour)
			{
				if (nr_args != 3) { error(arg_list, "Should have tree argument"); return new Expr(); }
				if (!assertIntType(arg_exprs[0], arg_list.part(1))) return new Expr();
				if (!assertIntType(arg_exprs[1], arg_list.part(2))) return new Expr();
				if (!assertDoubleColorType(arg_exprs[2], arg_list.part(3))) return new Expr();
				result = new ImageSetColourMethodExpr((VarImageExpr*)target, (IntExpr*)arg_exprs[0], (IntExpr*)arg_exprs[1], (DoubleColorExpr*)arg_exprs[2]);
			}
		}
		if (result == 0)
		{
			error(tree, "Unknown method");
			for (int i = 0; i < nr_args; i++)
				delete arg_exprs[i];
			delete arg_exprs;
			return new Expr();
		}
		delete arg_exprs;
		return result;
	}

	error(tree, "Not parsed yet");

	return new Expr();
}

/*
	Initialization of the IParse grammar
	~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	Below the procedure that initializes the Abstract Program Tree
	for the input grammar of IParse itself is given. This was
	generated by calling the procedure print_tree_to_c above.
*/

#include "ScriptGrammar.h"

void executeScript(const char* script)
{
	IParser* parser = new IParser();
	BasicScanner* scanner = new BasicScanner();
	parser->setScanner(scanner);
	parser->setDebugLevel(false, false, false);
	AbstractParseTree script_grammar_tree;
	init_ScriptGrammar(script_grammar_tree);
	parser->loadGrammer(script_grammar_tree);

	AbstractParseTree parse_tree;
	if (!parser->parse(script, "root", parse_tree))
	{
		TRACE("Failed");
		parser->traceExpected();
		char buffer[1000];
		buffer[0] = '\0';
		int len = 0;
		for (AbstractParser::ExpectAccessor expectAccessor(parser); expectAccessor.more(); expectAccessor.next())
		{
			if (len < 900)
			{
				sprintf(buffer+len, " %s", expectAccessor.symbol());
				len += strlen(buffer+len);
			}
		}
		long line;
		long column;
		parser->getExpectedPos(line, column);
		CString prompt;
		prompt.Format("%d,%d: Expected%s", line, column, buffer);
		AfxMessageBox(prompt);
		return;
	}

	Context* context = 0;
	Expr* statements = compile(parse_tree, context);
	if (statements != 0 && statements->type() == Expr::typeVoid)
		((VoidExpr*)statements)->eval();
}